xquery version "3.1";

module namespace resolve="https://mollenhauer-edition.de/ns/resolve";

declare namespace tei="http://www.tei-c.org/ns/1.0";

import module namespace config="https://sade.textgrid.de/ns/config" at "../../config.xqm";


(:~
 : This function copies a TEI-node.
 :
 : @param $node A TEI-node with an attribute copyOf referencing another node
 : @return The desired copy as a constructed node
 :)
declare function resolve:copy-node($node, $filename)
as node()*
{
    let $doc := doc($config:tei || $filename)
    let $target := resolve:find-node($node/@copyOf, $filename)
    return
        if ($target/@class = "error-message") then
            element span {
                attribute class {"error-message"},
                $target
            }
        else (
            let $attributes := $node/@*[local-name(.) != "copyOf"]
            let $attributes :=
            for $attribute in $attributes return
                attribute {local-name($attribute)} {$attribute}
            let $result :=
                element {node-name($node)} {
                    $attributes,
                    $target/node()
                }
            return
                $result
        )
};

(:~
 : This function resolves a tei:join according to its attributes result, scope,
 : and target.
 :
 : @param $node A tei:join
 : @return A constructed node, according to the specifications of the tei:join
 :)
declare function resolve:resolve-join($node as node(), $filename as xs:string)
as node()?
{
    let $result := $node/@result
    let $attributes := $node/@*[local-name(.) != "result" or "scope" or "target"]
    let $attributes :=
        for $attribute in $attributes return
            if (local-name($attribute) != "target" or "scope" or "result") then
                attribute {local-name($attribute)} {$attribute}
            else
                ()
    let $targets := tokenize($node/@target)
    return
        element {$result} {
            $attributes,
            for $target in $targets return
                let $resolved-node := resolve:find-node($target, $filename)
                return
                    if ($resolved-node/@class = "error-message") then
                        element span {
                            attribute class {"error-message"},
                            $target
                        }
                    else
                        if ( $node/@scope = "branches" ) then
                            ($resolved-node/node(), " ")
                        else
                            $resolved-node
        }
};

declare function resolve:find-node($target, $filename) {
    if (fn:starts-with($target, "textgrid:")) then
        let $uri := substring($target, 10, 5)
        let $doc := doc($config:tei || $uri || '.xml')
        let $target := substring-after($target, "#")
        return
            if ($doc//*[@xml:id = $target]) then
                $doc//*[@xml:id = $target]
            else
                element span {
                    attribute class {"error-message"},
                    "Error: Can't find corresponding text with tguri: " || $uri ||
                    " in the database. Maybe the text hasn't been been published yet?",
                    element br {}
                }
    else
        let $doc := doc($config:tei || $filename)
        let $target := replace($target, "#", "")
        return
            if (fn:exists($doc//*[@xml:id = $target])) then
                $doc//*[@xml:id = $target]
            else
                element span {
                    attribute class {"error-message"},
                    "Error: Can't find node with xml:id: " || $target,
                    element br {}
                }
};

declare function resolve:includes($filename)
{
    let $doc := doc($config:tei || $filename)
    return
        (for $node in $doc//tei:join return
            update replace $node with resolve:resolve-join($node, $filename),
        for $node in $doc//*[@copyOf] return
            update replace $node with resolve:copy-node($node, $filename))
};